import type { IConfigService } from "../../application/interface/config-service.interface.js";
import type { IConfig } from "../../application/interface/config.interface.js";
import type { IFileSystemService } from "../../application/interface/file-system-service.interface.js";
/**
 * Implementation of ConfigService that uses cosmiconfig for configuration management.
 * Cosmiconfig searches for configuration in standard locations and formats.
 */
export declare class CosmicConfigService implements IConfigService {
    /** File system service for file operations */
    readonly FILE_SYSTEM_SERVICE: IFileSystemService;
    private cachedConfig;
    private readonly EXPLORER;
    /**
     * Initializes a new instance of the CosmicConfigService.
     * @param fileSystemService
     */
    constructor(fileSystemService: IFileSystemService);
    /**
     * Clears all caches.
     */
    clearCaches(): void;
    /**
     * Checks if the configuration exists.
     * @returns Promise resolving to true if the configuration exists, false otherwise
     */
    exists(): Promise<boolean>;
    /**
     * Retrieves the current configuration.
     * @returns Promise resolving to the configuration object
     */
    get(): Promise<IConfig>;
    /**
     * Gets a specific property from the configuration.
     * @param property - The property key to retrieve
     * @returns Promise resolving to the value of the specified property
     */
    getProperty<K extends keyof IConfig>(property: K): Promise<IConfig[K]>;
    /**
     * Merges partial configuration with the existing configuration.
     * @param partial - Partial configuration to merge
     * @returns Promise that resolves when the merged configuration is saved
     */
    merge(partial: Partial<IConfig>): Promise<void>;
    /**
     * Saves the entire configuration.
     * @param config - The complete configuration to save
     * @returns Promise that resolves when the configuration is saved
     */
    set(config: IConfig): Promise<void>;
    /**
     * Sets a specific property in the configuration.
     * @param property - The property key to set
     * @param value - The value to assign to the property
     * @returns Promise that resolves when the updated configuration is saved
     */
    setProperty<K extends keyof IConfig>(property: K, value: IConfig[K]): Promise<void>;
    /**
     * Writes configuration to a file.
     * @param filepath - Path to write the configuration to
     * @param config - Configuration to write
     * @returns Promise that resolves when the file is written
     */
    private writeFile;
}
//# sourceMappingURL=cosmic-config.service.d.ts.map